@@ -51,12 +51,14 @@ const string
 	NCO "fullname"),
     titlePropertyName(
 	NIE "title"),
+    albumArtistPropertyName(
+	NMM_DRAFT "albumArtist"),
     albumTrackCountPropertyName(
 	NMM_DRAFT "albumTrackCount"),
     discNumberPropertyName(
 	NMM_DRAFT "setNumber"),
     discCountPropertyName(
-	NMM_DRAFT "setCount"),
+	NMM_DRAFT "setSize"),
 
     musicClassName(
 	NMM_DRAFT "MusicPiece"),
@@ -81,7 +83,9 @@ replaygain
 VBR detection
 */
 
-static const string genres[148] = {
+#define ID3_NUMBER_OF_GENRES 148
+
+static const string genres[ID3_NUMBER_OF_GENRES] = {
   "Blues",
   "Classic Rock",
   "Country",
@@ -290,7 +294,6 @@ ID3EndAnalyzerFactory::registerFields(FieldRegister& r) {
     titleField		= r.registerField(titlePropertyName);
     descriptionField	= r.registerField(NIE "description");
     commentField	= r.registerField(NIE "comment");
-    artistField		= r.registerField(NCO "creator");
     albumField		= r.registerField(NMM_DRAFT "musicAlbum");
     genreField		= r.registerField(NMM_DRAFT "genre");
     composerField	= r.registerField(NMM_DRAFT "composer");
@@ -300,6 +303,7 @@ ID3EndAnalyzerFactory::registerFields(FieldRegister& r) {
     languageField	= r.registerField(NIE "language");
     copyrightField	= r.registerField(NIE "copyright");
     trackNumberField	= r.registerField(NMM_DRAFT "trackNumber");
+    discNumberField	= r.registerField(discNumberPropertyName);
     durationField	= r.registerField(NFO "duration");
     typeField		= r.typeField;
 
@@ -336,15 +340,18 @@ readSize(const unsigned char* b, bool async) {
 bool
 ID3EndAnalyzer::checkHeader(const char* header, int32_t headersize) const {
   const unsigned char* usbuf = (const unsigned char*)header;
-  return (headersize>=6)
+  int32_t i;
+
+  for(i=0; (header[i] == '\0') && (i<headersize); i++);
+  return (headersize>=6+i)
 	  && (
-	    (strncmp("ID3", header, 3) == 0	// check that it's ID3
-	      && usbuf[3] <= 4 				// only handle version <= 4
-	      && header[5] == 0)  // we're too dumb too handle flags
+	    (strncmp("ID3", header+i, 3) == 0	// check that it's ID3
+	      && usbuf[3+i] <= 4 				// only handle version <= 4
+	      && (usbuf[5+i]&~0x80) == 0)  // we're too dumb too handle other flags
 	    ||
-	    ((unsigned char)header[0] == 0xff && ((unsigned char)header[1]&0xfe) == 0xfa
-	      && (unsigned char)header[2]>>4 != 0xf	// MP3 frame header is ok too
-	      && (((unsigned char)header[2]>>2)&3) != 3)
+	    ((unsigned char)header[0+i] == 0xff && ((unsigned char)header[1+i]&0xfe) == 0xfa
+	      && (unsigned char)header[2+i]>>4 != 0xf	// MP3 frame header is ok too
+	      && (((unsigned char)header[2+i]>>2)&3) != 3)
 	  );
 
 }
@@ -369,8 +376,57 @@ static bool extract_and_trim(const char* buf, int offset, int length, string& s)
     return !s.empty();
 }
 
+/**
+ * Functional helper class to get the right numbers out of a 'genre' string which
+ * might be a number in a index
+ */
+class genre_number_parser {
+  private:
+    bool success;
+    long result;
+    void parse_string( string genre ) {
+        char* endptr;
+        int r = strtol(genre.c_str(),&endptr, 10);
+        if(*endptr == '\0') { //to check if the convertion went more or less ok
+	    if(r >=0 && r < ID3_NUMBER_OF_GENRES ) { //to ensure it is within the range we have
+	        success=true;
+	        result=r;
+	    }
+        }
+    }
+  public:
+    /**
+     * constructor taking the genre string you want parsed as a number
+     */
+    genre_number_parser(string genre) : success(false), result(-1) {
+        if(genre.size()==0) {
+	  //if the string is empty, there is no need to try to parse it
+	    return;
+        }
+        //the string might start and end with parenthesis
+        if(genre[0]=='(' && genre[genre.size()-1]==')') {
+	    parse_string(genre.substr(1,genre.length()-2));
+	    return;
+        }
+        parse_string(genre);
+    }
+    /**
+     * wether or not parsing was successful
+     */
+    operator bool() {
+        return success;
+    }
+    /**
+     * the actual result of the parsing, or -1 if parsing wasn't successful
+     */
+    operator long() {
+        return result;
+    }
+};
+
 signed char
 ID3EndAnalyzer::analyze(Strigi::AnalysisResult& indexable, Strigi::InputStream* in) {
+  const int max_padding = 1000;
     if(!in)
         return -1;
 
@@ -383,19 +439,20 @@ ID3EndAnalyzer::analyze(Strigi::AnalysisResult& indexable, Strigi::InputStream*
 
     // read 10 byte header
     const char* buf;
-    int32_t nread = in->read(buf, 10, 10);
+    int32_t nread = in->read(buf, 10+max_padding, 10+max_padding);
 
     // parse ID3v2* tag
 
-    if (nread == 10 && strncmp("ID3", buf, 3) == 0) { // check for ID3 header
+    if (nread == 10+max_padding && strncmp("ID3", buf, 3) == 0) { // check for ID3 header
 
 	bool async = buf[3] >= 4;
+	bool unsync = (buf[5] & 0x80)!=0;
 
 	// calculate size from 4 syncsafe bytes
 	int32_t size = readAsyncSize((unsigned char*)buf+6);
-	if (size < 0 || size > 300000)
+	if (size < 0 || size > 5000000)
 	    return -1;
-	size += 10+4; // add the size of the ID3 header and MP3 frame header
+	size += 10+4+max_padding; // add the size of the ID3 header, MP3 frame header and possible padding generated by LAME(workaround)
 
 	// read the entire tag
 	in->reset(0);
@@ -406,7 +463,7 @@ ID3EndAnalyzer::analyze(Strigi::AnalysisResult& indexable, Strigi::InputStream*
 	found_tag = true;
 
 	const char* p = buf + 10;
-	buf += size-4;
+	buf += size-4-max_padding;
 	while (p < buf && *p) {
 	    size = readSize((unsigned char*)p+4, async);
 	    if (size <= 0 || size > (buf-p)-10) {
@@ -418,18 +475,32 @@ ID3EndAnalyzer::analyze(Strigi::AnalysisResult& indexable, Strigi::InputStream*
 	    uint8_t enc = p[10];
 	    const char *encoding = enc>4 ? encodings[0] : encodings[enc] ;
 	    UTF8Convertor conv(encoding);
+	    const char *decoded_value;
+	    int32_t decoded_value_size;
+	    string deunsyncbuf;
+	    if (unsync) {
+	      deunsyncbuf.reserve(size-1);
+	      for(int32_t i = 0; i<size-1; i++)
+		if ( (i==0) || (p[11+i]!=0) || (p[10+i]!=0xff) )
+		  deunsyncbuf.push_back(p[11+i]);
+	      decoded_value = deunsyncbuf.c_str();
+	      decoded_value_size = deunsyncbuf.length();
+	    } else {
+	      decoded_value = p+11;
+	      decoded_value_size = size-1;
+	    };
 
 	    if (strncmp("APIC", p, 4) == 0) {
-		size_t mimelen = strnlen(p+11, size-1);
-                if ((int32_t)mimelen < size-1-3) {
-		    const char *desc = p+11+mimelen+1+1;
+		size_t mimelen = strnlen(decoded_value, decoded_value_size);
+                if ((int32_t)mimelen < decoded_value_size-3) {
+		    const char *desc = decoded_value+mimelen+1+1;
 //		    uint8_t pictype = p[11+mimelen+1];
-		    size_t desclen = strnlen(desc,size-1-mimelen-2-1);
+		    size_t desclen = strnlen(desc,decoded_value_size-mimelen-2-1);
 		    const char *content = desc + desclen + 1 + (enc == 0 || enc == 3 ? 0:1) ;
 
-		    if(content<p+10+size) {
+		    if(content<decoded_value+decoded_value_size) {
                         StringInputStream picstream(content,
-                                          (uint32_t)(p+10+size-content), false);
+                                          (uint32_t)(decoded_value+decoded_value_size-content), false);
 			string picname;
 			picname = (char)('0'+albumArtNum++);
 			indexable.indexChild(picname, indexable.mTime(), &picstream);
@@ -448,9 +519,9 @@ ID3EndAnalyzer::analyze(Strigi::AnalysisResult& indexable, Strigi::InputStream*
 	    }
 
 	    if (enc == 0 || enc == 3) {
-		value = string(p+11, strnlen(p+11, size-1));
+		value = string(decoded_value, strnlen(decoded_value, decoded_value_size));
 	    } else {
-		value = conv.convert(p+11,size-1); // FIXME: add similar workaround
+		value = conv.convert(decoded_value, decoded_value_size); // FIXME: add similar workaround
 	    }
 
 	    if (!value.empty()) {
@@ -472,7 +543,6 @@ ID3EndAnalyzer::analyze(Strigi::AnalysisResult& indexable, Strigi::InputStream*
 		    indexable.addValue(factory->createdField, value);
 		    found_year = true;
 		} else if ((strncmp("TPE1", p, 4) == 0) ||
-			    (strncmp("TPE2", p, 4) == 0) ||
 			    (strncmp("TPE3", p, 4) == 0) ||
 			    (strncmp("TPE4", p, 4) == 0)) {
 		    string performerUri = indexable.newAnonymousUri();
@@ -481,6 +551,12 @@ ID3EndAnalyzer::analyze(Strigi::AnalysisResult& indexable, Strigi::InputStream*
 		    indexable.addTriplet(performerUri, typePropertyName, contactClassName);
 		    indexable.addTriplet(performerUri, fullnamePropertyName, value);
 		    found_artist = true;
+		} else if (strncmp("TPE2", p, 4) == 0) {
+		    const string albumArtistUri( indexable.newAnonymousUri() );
+
+		    addStatement(indexable, albumUri, albumArtistPropertyName, albumArtistUri);
+		    indexable.addTriplet(albumArtistUri, typePropertyName, contactClassName);
+		    indexable.addTriplet(albumArtistUri, fullnamePropertyName, value);
 		} else if ((strncmp("TPUB", p, 4) == 0) ||
 			    (strncmp("TENC", p, 4) == 0)) {
 		    string publisherUri = indexable.newAnonymousUri();
@@ -493,8 +569,17 @@ ID3EndAnalyzer::analyze(Strigi::AnalysisResult& indexable, Strigi::InputStream*
 		    addStatement(indexable, albumUri, titlePropertyName, value);
 		    found_album = true;
 		} else if (strncmp("TCON", p, 4) == 0) {
-		    indexable.addValue(factory->genreField, value);
-		    found_genre = true;
+		    genre_number_parser p(value);
+		    if(p) {
+			indexable.addValue(factory->genreField, genres[ p ]);
+			found_genre = true;
+		    } else {
+			// We must not forget that genre could be a string.
+			if (!value.empty()) {
+			    indexable.addValue(factory->genreField, value);
+			    found_genre = true;
+			}
+		    }
 		} else if (strncmp("TLEN", p, 4) == 0) {
 		    indexable.addValue(factory->durationField, value);
 		} else if (strncmp("TEXT", p, 4) == 0) {
@@ -530,9 +615,7 @@ ID3EndAnalyzer::analyze(Strigi::AnalysisResult& indexable, Strigi::InputStream*
 		    int dnum;
 		    ins >> dnum;
 		    if (!ins.fail()) {
-			ostringstream out;
-			out << dnum;
-			addStatement(indexable, albumUri, discNumberPropertyName, out.str());
+			indexable.addValue(factory->discNumberField, dnum);
 			ins.ignore(10,'/');
 			int dcount;
 			ins >> dcount;
@@ -550,9 +633,11 @@ ID3EndAnalyzer::analyze(Strigi::AnalysisResult& indexable, Strigi::InputStream*
     // parse MP3 frame header
 
     int bitrateindex, samplerateindex;
-    if (((unsigned char)buf[0] == 0xff) && (((unsigned char)buf[1]&0xfe) == 0xfa)
-      && ((bitrateindex = ((unsigned char)buf[2]>>4)) != 0xf)
-      && ((samplerateindex = (((unsigned char)buf[2]>>2)&3)) != 3 )) { // is this MP3?
+    int i;
+    for(i=0; (buf[i]=='\0') && (i<max_padding) ; i++);
+    if (((unsigned char)buf[0+i] == 0xff) && (((unsigned char)buf[1+i]&0xfe) == 0xfa)
+      && ((bitrateindex = ((unsigned char)buf[2+i]>>4)) != 0xf)
+      && ((samplerateindex = (((unsigned char)buf[2+i]>>2)&3)) != 3 )) { // is this MP3?
 
 	indexable.addValue(factory->typeField, audioClassName);
 	// FIXME: no support for VBR :(
@@ -562,7 +647,7 @@ ID3EndAnalyzer::analyze(Strigi::AnalysisResult& indexable, Strigi::InputStream*
 	indexable.addValue(factory->bitrateField, bitrate[bitrateindex]);
 	indexable.addValue(factory->samplerateField, samplerate[samplerateindex]);
 	indexable.addValue(factory->codecField, "MP3");
-	indexable.addValue(factory->channelsField, ((buf[3]>>6) == 3 ? 1:2 ) );
+	indexable.addValue(factory->channelsField, ((buf[3+i]>>6) == 3 ? 1:2 ) );
     }
 
     // Parse ID3v1 tag
@@ -583,8 +668,12 @@ ID3EndAnalyzer::analyze(Strigi::AnalysisResult& indexable, Strigi::InputStream*
 	    if (!found_title && extract_and_trim(buf, 3, 30, s)) {
 		indexable.addValue(factory->titleField, s);
 	    }
-	    if (!found_artist && extract_and_trim(buf, 33, 30, s))
-		indexable.addValue(factory->artistField, s);
+	    if (!found_artist && extract_and_trim(buf, 33, 30, s)) {
+                const string performerUri = indexable.newAnonymousUri();
+                indexable.addValue(factory->performerField, performerUri);
+                indexable.addTriplet(performerUri, typePropertyName, contactClassName);
+                indexable.addTriplet(performerUri, fullnamePropertyName, s);
+            }
 	    if (!found_album && extract_and_trim(buf, 63, 30, s))
 		addStatement(indexable, albumUri, titlePropertyName, s);
 	    if (!found_year && extract_and_trim(buf, 93, 4, s))
@@ -595,7 +684,7 @@ ID3EndAnalyzer::analyze(Strigi::AnalysisResult& indexable, Strigi::InputStream*
 	    if (!found_track && !buf[125] && buf[126]) {
 		indexable.addValue(factory->trackNumberField, (int)(buf[126]));
 	    }
-	    if (!found_genre && (unsigned char)(buf[127]) < 148)
+	    if (!found_genre && (unsigned char)(buf[127]) < ID3_NUMBER_OF_GENRES)
 		indexable.addValue(factory->genreField, genres[(uint8_t)buf[127]]);
 	}
     }
